@@ -98,6 +98,8 @@ int target_is_native (struct target_ops *t);
 
 int gdbtk_test (char *);
 
+static void view_command (char *, int);
+
 /* Handle for TCL interpreter */
 Tcl_Interp *gdbtk_interp = NULL;
 
@@ -568,6 +570,9 @@ gdbtk_init (char *argv0)
   add_com ("tk", class_obscure, tk_command,
 	   "Send a command directly into tk.");
 
+  add_com ("view", class_obscure, view_command,
+	   "View a location in the source window.");
+
   /*
    * Set the variable for external editor:
    */
@@ -635,12 +640,16 @@ gdbtk_find_main";
 	msg = Tcl_GetVar (gdbtk_interp, "errorInfo", TCL_GLOBAL_ONLY);
 
 #ifdef _WIN32
+	/* On windows, display the error using a pop-up message box.
+           If GDB wasn't started from the DOS prompt, the user won't
+           get to see the failure reason.  */
 	MessageBox (NULL, msg, NULL, MB_OK | MB_ICONERROR | MB_TASKMODAL);
+	throw_exception (RETURN_ERROR);
 #else
-	fprintf (stderr,msg);
+	/* FIXME: cagney/2002-04-17: Wonder what the lifetime of
+           ``msg'' is - does it need a cleanup?  */
+	error (msg);
 #endif
-
-	error ("");
       }
   }
 
@@ -737,3 +746,27 @@ tk_command (char *cmd, int from_tty)
 
   do_cleanups (old_chain);
 }
+
+static void
+view_command (char *args, int from_tty)
+{
+  char *script;
+  struct cleanup *old_chain;
+
+  if (args != NULL)
+    {
+      xasprintf (&script,
+		 "[lindex [ManagedWin::find SrcWin] 0] location BROWSE_TAG [gdb_loc %s]",
+		 args);
+      old_chain = make_cleanup (xfree, script);
+      if (Tcl_Eval (gdbtk_interp, script) != TCL_OK)
+	{
+	  Tcl_Obj *obj = Tcl_GetObjResult (gdbtk_interp);
+	  error (Tcl_GetStringFromObj (obj, NULL));
+	}
+
+      do_cleanups (old_chain);
+    }
+  else
+    error ("Argument required (location to view)");
+}
